﻿from ctypes import *
import ctypes
import time

#AqLAVISA constants
#Error code
AQVI_DLL_NOT_READY = 9999

#viSelectAppType options
APP_TYPE_TL = 0 #default
APP_TYPE_BF = 1
APP_TYPE_TB = 2
APP_TYPE_MS = 3

#VISA Get Command Result
PROC_OK_WOUT_RETURN = 0
PROC_OK_WITH_RETURN = 1
PROC_FAIL = 2
PROC_PENDING = 3

#Environment Settings
TargetAppType = APP_TYPE_BF
AqLAVISA_DLL_Directory = "D:\\AqVISA64.dll"
ReportCsvSaveDirectory = b"D:\\Temp\\myReport.csv"

#AqLAVISA class define
class AqLAVISA:
    #constructor
    def __init__(self, DllDirectory):
        self.DllDirectory = DllDirectory
        self.LaDll = ctypes.WinDLL(DllDirectory)
        self.LaDll.viWrite.argtypes = [ctypes.c_char_p]
        self.LaDll.viRead.argtypes = [ctypes.c_char_p, ctypes.c_int]

    #Select Target Application Type
    def viSelectAppType(self, AppType):
        RetryCount = 10 
        while (self.LaDll.viSelectAppType(AppType) == 0):
            if (self.LaDll.viErrCode() != AQVI_DLL_NOT_READY): #Error code = 9999, DLL still initializing
                print ("viSelectAppType Error... ", self.LaDll.viErrCode())
                return False
            if (RetryCount == 0):
                print ("viSelectAppType Timeout... ", self.LaDll.viErrCode())
                return False
            RetryCount -= 1
            time.sleep(0.1) #Wait 100ms for each loop	
        return True
                
    #Establish S/W connection
    def viOpenRM(self):
        RetryCount = 10 
        while (self.LaDll.viOpenRM() == 0):
            if (self.LaDll.viErrCode() != AQVI_DLL_NOT_READY): #Error code = 9999, DLL still initializing
                print ("viOpenRM Error... ", self.LaDll.viErrCode())
                return False
            if (RetryCount == 0):
                print ("viOpenRM Timeout... ", self.LaDll.viErrCode())
                return False
            RetryCount -= 1
            time.sleep(0.1) #Wait 100ms for each loop	
        return True

    #Read Error Code
    def viErrCode(self):
        return self.LaDll.viErrCode()
        
    #Send Command to S/W
    def viWrite(self, cmd, timeout = 10):      
        if (self.LaDll.viWrite(ctypes.c_char_p(cmd)) == 0):
            return False
        TimerInterval = 0.1 #100ms
        RetryCount = timeout / TimerInterval
        CmdResult = self.LaDll.viGetCommandResult()
        while (CmdResult == PROC_PENDING and RetryCount > 0):
            time.sleep(TimerInterval) #Wait 100ms for command processing
            CmdResult = self.LaDll.viGetCommandResult()
            RetryCount -= 1
        if (CmdResult == PROC_FAIL):
            print("viWrite Error...", self.LaDll.viErrCode(), cmd)
            return False
        if (CmdResult == PROC_PENDING):
            print("viWrite Timeout...", self.LaDll.viErrCode(), cmd)
            return False
        return True

    #Read Command Return from S/W
    def viRead(self):
        BufSize = 1024
        ReadBuf = ctypes.create_string_buffer(BufSize)
        if (self.LaDll.viRead(ReadBuf, BufSize) == 0):
            print ("viRead Error... ", self.LaDll.viErrCode())
            return ""
        else: 
            return ReadBuf.value

    #Disconnect from S/W
    def viCloseRM(self):
        if (self.LaDll.viCloseRM() == 0):
            print ("viCloseRM Error... ", self.LaDll.viErrCode())



#############################################



titles = []

def CaptureAndSaveReport():    

    LaVisaDll = AqLAVISA(AqLAVISA_DLL_Directory)    

    if (LaVisaDll.viSelectAppType(TargetAppType) == 0):
        return

    print ("Initial software connection...")
    if (LaVisaDll.viOpenRM() == 0):
        return
        
    print ("Begin software capture...")
    if (LaVisaDll.viWrite(b"*PA:CAPTURE:START") == 0): #Begin Capture
        return
    
    print ("Wait for waveform capture...")
    time.sleep(10) #Wait 10s for waveform capture    

    print ("Stop software capture, and wait until software ready")
    if (LaVisaDll.viWrite(b"*PA:CAPTURE:STOP") == 0): #Stop Capture
        return
        
    while (1): #wait until capture finished
        if (LaVisaDll.viWrite(b"*STB?") == 0): #Status check            
            return 
        ReadString = LaVisaDll.viRead()
        if (ReadString == ""):
            return
        if (ReadString == b'1'):
            break
        
    print ("Begin save captured data as report...")
    SaveCSVWaitTime = 30 #wait 30s for file saving
    if (LaVisaDll.viWrite(b"*PA:REPORT:SAVE " + ReportCsvSaveDirectory, SaveCSVWaitTime) == 0): #Save Report
        return
    while (1): #wait until report save finished
        if (LaVisaDll.viWrite(b"*STB?") == 0): #Status check
            return
        ReadString = LaVisaDll.viRead()
        if (ReadString == ""):            
            return
        if (ReadString == b'1'):
            break

    print ("Disconnect from software...")
    LaVisaDll.viCloseRM() 
    print ("All Finished!")
    
def main():
    CaptureAndSaveReport()
   
if __name__ == "__main__":
    main()
